=== modified file '__init__.py'
--- __init__.py	2007-11-23 15:13:59 +0000
+++ __init__.py	2007-12-20 18:29:24 +0000
@@ -438,12 +438,16 @@
     deprecated.
     """
     takes_options = [Option("lightweight",
-                            help="Create a lightweight checkout."), 'revision']
+                            help="Create a lightweight checkout."), 'revision',
+                     Option('from-files', type=unicode,
+                            help='Accelerate checkout using files from this'
+                                 ' tree.')]
     takes_args = ["source", "target?"]
-    def run(self, source, target=None, lightweight=False, revision=None):
+    def run(self, source, target=None, lightweight=False, revision=None,
+            from_files=None):
         from cbranch import cbranch
         return cbranch(source, target, lightweight=lightweight,
-                       revision=revision)
+                       revision=revision, from_files=from_files)
 
 
 class cmd_branches(BzrToolsCommand):

=== modified file 'cbranch.py'
--- cbranch.py	2007-01-04 15:37:11 +0000
+++ cbranch.py	2007-12-20 18:29:24 +0000
@@ -15,18 +15,19 @@
 #    along with this program; if not, write to the Free Software
 #    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+from bzrlib.bzrdir import BzrDir
 from bzrlib.branch import Branch
-from bzrlib.builtins import cmd_branch, cmd_checkout
 from bzrlib.config import LocationConfig
 from bzrlib.errors import BzrCommandError
-from bzrlib.osutils import pathjoin, basename, abspath, getcwd
-from bzrlib.urlutils import local_path_to_url
+from bzrlib.osutils import pathjoin, basename, abspath
+from bzrlib.workingtree import WorkingTree
+from bzrlib import ui
+from bzrlib.urlutils import derive_to_location
 
 def cbranch(from_location, to_location=None, revision=None,
-            lightweight=False):
-    from_location = from_location.rstrip('/')
+            lightweight=False, from_files=None):
     if to_location is None:
-        to_location = pathjoin(getcwd(), basename(from_location))
+        to_location = derive_to_location(from_location)
     config = LocationConfig(abspath(to_location))
     b_loc = config.get_user_option("cbranch_target")
     if b_loc is None:
@@ -35,6 +36,22 @@
             raise BzrCommandError("Can't find cbranch_target in"
                                   " locations.conf")
         b_loc = pathjoin(b_root, basename(to_location))
-    cmd_branch().run(from_location, b_loc, revision=revision)
-    cmd_checkout().run(b_loc, to_location, lightweight=lightweight)
-    b = Branch.open(to_location)
+    accelerator_tree, old_branch = BzrDir.open_tree_or_branch(from_location)
+    if from_files is not None:
+        accelerator_tree = WorkingTree.open(from_files)
+    if revision is None or len(revision) == 0:
+        revision_id = old_branch.last_revision()
+    elif len(revision) == 1:
+        revision_id = revision[0].in_history(old_branch)[1]
+    else:
+        raise BzrCommandError('At most one revision may be supplied.')
+    pb = ui.ui_factory.nested_progress_bar()
+    try:
+        pb.update('Creating branch', 0, 2)
+        new_branch = old_branch.bzrdir.sprout(b_loc, revision_id,
+            accelerator_tree=accelerator_tree).open_branch()
+        pb.update('Creating checkout', 1, 2)
+        new_branch.create_checkout(to_location, lightweight=lightweight,
+                                   accelerator_tree=accelerator_tree)
+    finally:
+        pb.finished()

